#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ts.h" 

void inicializa_ts(ts_t *ts){
	ts->topo = NULL;
	ts->tam = 0;
}

void insere_ts(char *ident, int categ, int nivel, info_t info, ts_t *ts){
	if(ts == NULL) return;

	simb_t *new = (simb_t *)malloc(sizeof(simb_t));
	new->ident = (char *)malloc((strlen(ident) + 1) * sizeof(char));
	strcpy(new->ident, ident);
	new->categ = categ;
	new->nivel = nivel;
    if (categ == VS) {
        new->info.vs.tipo = info.vs.tipo;
        new->info.vs.desloc = info.vs.desloc;
    } else if (categ == PR) {
        strcpy(new->info.pr.rot, info.pr.rot);
        new->info.pr.quant = info.pr.quant;
        new->info.pr.param = (int **)malloc(info.pr.quant * sizeof(int *));
        for (int i = 0; i < info.pr.quant; i++) {
            new->info.pr.param[i] = (int *)malloc(2 * sizeof(int));
            new->info.pr.param[i][0] = info.pr.param[i][0];
            new->info.pr.param[i][1] = info.pr.param[i][1];
        }
    } else if (categ == PF) {
        new->info.pf.tipo = info.pf.tipo;
        new->info.pf.desloc = info.pf.desloc;
        new->info.pf.passagem = info.pf.passagem;
    } else if (categ == FUN) {
        strcpy(new->info.fun.rot, info.fun.rot);
        new->info.fun.quant = info.fun.quant;
        new->info.fun.param = (int **)malloc(info.fun.quant * sizeof(int *));
        for (int i = 0; i < info.fun.quant; i++) {
            new->info.fun.param[i] = (int *)malloc(2 * sizeof(int));
            new->info.fun.param[i][0] = info.fun.param[i][0];
            new->info.fun.param[i][1] = info.fun.param[i][1];
        }
        new->info.fun.tipo = info.fun.tipo;
        new->info.fun.desloc = info.fun.desloc;
    }

	new->prox = ts->topo;
	ts->topo = new;
	ts->tam += 1;
}

simb_t *busca(char *ident, ts_t *ts){
	if(ts == NULL) return NULL;

	simb_t *p = ts->topo; //ponteiro?
	if (strcmp(p->ident, ident) == 0) return p; //str compare
	for (int i = 1; i < ts->tam; ++i){
		p = p->prox;
		if (strcmp(p->ident, ident) == 0) return p; //str compare
	}
	return NULL;//nao tem
}

simb_t *topo(ts_t *ts){
    if(ts == NULL) return NULL;
    return ts->topo;
}

int retira_vs_pf(ts_t *ts) {
    if (ts == NULL || ts->topo == NULL) {
        return 0; // Nenhum elemento a ser removido
    }

    int i; // Contador de elementos removidos
    simb_t *temp = ts->topo;
    while (ts->topo != NULL && (ts->topo->categ == VS || ts->topo->categ == PF)) {
        temp = ts->topo;  
        ts->topo = ts->topo->prox; 
        //free the intern stuff
        if (temp->categ == VS) {
            free(temp->ident);
        } else if (temp->categ == PF) {
            free(temp->ident);
        }
        free(temp); 
        ts->tam--;  
    }

    //retorna novo deslocamento
    while (temp != NULL) {
        if (temp->categ == VS || temp->categ == PF) {
            return temp->info.vs.desloc + 1;
        }
        temp = temp->prox;
    }
    return 0;
}

int retira_pr_fun(ts_t *ts) {
    if (ts == NULL || ts->topo == NULL) {
        return 0; // Nenhum elemento a ser removido
    }

    int i; // Contador de elementos removidos
    simb_t *temp = ts->topo;
    while (ts->topo != NULL && (ts->topo->categ == PR || ts->topo->categ == FUN)) {
        temp = ts->topo;  
        ts->topo = ts->topo->prox; 
        //free the intern stuff
        if (temp->categ == PR) {
            free(temp->ident);
            for (int i = 0; i < temp->info.pr.quant; i++) {
                free(temp->info.pr.param[i]);
            }
            free(temp->info.pr.param);
        } else if (temp->categ == FUN) {
            free(temp->ident);
            for (int i = 0; i < temp->info.fun.quant; i++) {
                free(temp->info.fun.param[i]);
            }
            free(temp->info.fun.param);
        }
        free(temp); 
        ts->tam--;  
    }

    //retorna novo deslocamento
    while (temp != NULL) {
        if (temp->categ == VS || temp->categ == PF) {
            return temp->info.vs.desloc + 1;
        }
        temp = temp->prox;
    }
    return 0;
}

int retira(int n, ts_t *ts) {
    if (ts == NULL || ts->topo == NULL) {
        return 0; // Nenhum elemento a ser removido
    }

    int i; // Contador de elementos removidos
    simb_t *temp = ts->topo;
    for (i = 0; i < n && ts->topo != NULL; i++) {
        temp = ts->topo;  
        ts->topo = ts->topo->prox; 
        free(temp); 
        ts->tam--;  
    }

    if (i < n) {
        printf("Aviso: Tentativa de remover mais itens (%d) do que a pilha possui (%d). Removidos apenas %d itens.\n", n, i, i);
    }

    //retorna novo deslocamento
    while (temp != NULL) {
        if (temp->categ == VS) {
            return temp->info.vs.desloc + 1;
        }
        temp = temp->prox;
    }
    return 0;
}

void atualiza_tipo(int n, int tipo, ts_t *ts){
	if(ts == NULL || ts->topo == NULL || n <= 0) return;

	simb_t *p = ts->topo;
    int count = 0;
    while (p != NULL && count < n) {
        if (p->categ == VS && p->info.vs.tipo == NSEI) {
            p->info.vs.tipo = tipo;
            count++;
        } else if (p->categ == PF && p->info.pf.tipo == NSEI) {
            p->info.pf.tipo = tipo;
            count++;
        } else if (p->categ == FUN && p->info.fun.tipo == NSEI) {
            p->info.fun.tipo = tipo;
            count++;
        }
        p = p->prox;
    }
}

void adiciona_param(char *ident, int n, ts_t *ts){
    if(ts == NULL || ts->topo == NULL || n <= 0) return;

    // Add n parameters to ident procedure params
    simb_t *p = busca(ident, ts);
    if (p == NULL || (p->categ != PR && p->categ != FUN)) return;

    if (p->categ == PR) {
        if(p->info.pr.quant != 0) return;

        p->info.pr.param = (int **)malloc(n * sizeof(int *));
        p->info.pr.quant = n;

        simb_t *pf = ts->topo;
        for (int i = n-1, j = 0; i >= 0 && pf != NULL; i--, j++) {
            if (pf->categ == PF) {
                p->info.pr.param[i] = (int *)malloc(2 * sizeof(int));
                p->info.pr.param[i][0] = pf->info.pf.tipo;
                p->info.pr.param[i][1] = pf->info.pf.passagem;
                pf->info.pf.desloc = -4 - j;
            }
            pf = pf->prox;
        }
    } else if (p->categ == FUN) {
        if(p->info.fun.quant != 0) return;

        p->info.fun.param = (int **)malloc(n * sizeof(int *));
        p->info.fun.quant = n;

        simb_t *pf = ts->topo;
        int i,j;
        for (i = n-1, j = 0; i >= 0 && pf != NULL; i--, j++) {
            if (pf->categ == PF) {
                p->info.fun.param[i] = (int *)malloc(2 * sizeof(int));
                p->info.fun.param[i][0] = pf->info.pf.tipo;
                p->info.fun.param[i][1] = pf->info.pf.passagem;
                pf->info.pf.desloc = -4 - j;
            }
            pf = pf->prox;
        }
        p->info.fun.desloc = -4 - j;
    }
}

void mostra_ts(ts_t *ts) {
    if (ts == NULL || ts->topo == NULL) {
        printf("Tabela de Símbolos está vazia ou não inicializada.\n");
        return;
    }

    printf("\nTabela de Símbolos:\n");
    printf("----------------------------------------------------------------------------------\n");
    printf("| %-10s | %-8s | %-8s | %-10s | %-10s | %-20s |\n", 
           "Ident", "Categ", "Nivel", "Tipo", "Desloc", "Detalhes");
    printf("----------------------------------------------------------------------------------\n");

    simb_t *p = ts->topo;
    while (p != NULL) {
        const char *categStr = "Outro";
        const char *tipoStr = "-";
        char deslocStr[10] = "-";
        char detalhes[200] = "-";

        switch (p->categ) {
            case VS: {
                categStr = "VS";
                switch (p->info.vs.tipo) {
                    case INTEIRO:
                        tipoStr = "Inteiro";
                        break;
                    case BOOLEAN:
                        tipoStr = "Boolean";
                        break;
                    default:
                        tipoStr = "NSEI";
                        break;
                }
                snprintf(deslocStr, sizeof(deslocStr), "%d", p->info.vs.desloc);
                break;
            }
            case PR: {
                categStr = "PR";
                tipoStr = "-";
                if (p->info.pr.quant > 1) {
                    char paramStr[150] = "";
                    snprintf(paramStr, sizeof(paramStr), "%d{", p->info.pr.quant);
                    for (int i = 0; i < p->info.pr.quant; i++) {
                        const char *paramTipo = 
                            (p->info.pr.param[i][0] == INTEIRO) ? "Int" :
                            (p->info.pr.param[i][0] == BOOLEAN) ? "Bool" : "NSEI";
                        const char *paramPassagem = 
                            (p->info.pr.param[i][1] == VLR) ? "VLR" : "REF";

                        char singleParam[30];
                        snprintf(singleParam, sizeof(singleParam), "[%s,%s]", paramTipo, paramPassagem);
                        strncat(paramStr, singleParam, sizeof(paramStr) - strlen(paramStr) - 1);
                        if (i < p->info.pr.quant - 1) {
                            strncat(paramStr, ",", sizeof(paramStr) - strlen(paramStr) - 1);
                        }
                    }
                    strncat(paramStr, "}", sizeof(paramStr) - strlen(paramStr) - 1);
                    snprintf(detalhes, sizeof(detalhes), "%s, Params: %s", p->info.pr.rot, paramStr);
                } else {
                    snprintf(detalhes, sizeof(detalhes), "%s, Params: %d", p->info.pr.rot, p->info.pr.quant);
                }
                break;
            }
            case PF: {
                categStr = "PF";
                switch (p->info.pf.tipo) {
                    case INTEIRO:
                        tipoStr = "Inteiro";
                        break;
                    case BOOLEAN:
                        tipoStr = "Boolean";
                        break;
                    default:
                        tipoStr = "NSEI";
                        break;
                }
                snprintf(deslocStr, sizeof(deslocStr), "%d", p->info.pf.desloc);
                snprintf(detalhes, sizeof(detalhes), "Passagem: %s", p->info.pf.passagem == VLR ? "Valor" : "Referência");
                break;
            }
            case FUN: {
                categStr = "FUN";
                switch (p->info.fun.tipo) {
                    case INTEIRO:
                        tipoStr = "Inteiro";
                        break;
                    case BOOLEAN:
                        tipoStr = "Boolean";
                        break;
                    default:
                        tipoStr = "NSEI";
                        break;
                }
                snprintf(deslocStr, sizeof(deslocStr), "%d", p->info.fun.desloc);
                if (p->info.fun.quant > 1) {
                    char paramStr[150] = "";
                    snprintf(paramStr, sizeof(paramStr), "%d{", p->info.fun.quant);
                    for (int i = 0; i < p->info.fun.quant; i++) {
                        const char *paramTipo = 
                            (p->info.fun.param[i][0] == INTEIRO) ? "Int" :
                            (p->info.fun.param[i][0] == BOOLEAN) ? "Bool" : "NSEI";
                        const char *paramPassagem = 
                            (p->info.fun.param[i][1] == VLR) ? "VLR" : "REF";

                        char singleParam[30];
                        snprintf(singleParam, sizeof(singleParam), "[%s,%s]", paramTipo, paramPassagem);
                        strncat(paramStr, singleParam, sizeof(paramStr) - strlen(paramStr) - 1);
                        if (i < p->info.fun.quant - 1) {
                            strncat(paramStr, ",", sizeof(paramStr) - strlen(paramStr) - 1);
                        }
                    }
                    strncat(paramStr, "}", sizeof(paramStr) - strlen(paramStr) - 1);
                    snprintf(detalhes, sizeof(detalhes), "%s, Params: %s", p->info.fun.rot, paramStr);
                } else {
                    snprintf(detalhes, sizeof(detalhes), "%s, Params: %d", p->info.fun.rot, p->info.fun.quant);
                }
                break;
            }
            default:
                categStr = "Outro";
        }

        printf("| %-10s | %-8s | %-8d | %-10s | %-10s | %-20s |\n", 
               p->ident, categStr, p->nivel, tipoStr, deslocStr, detalhes);

        p = p->prox;
    }
    printf("----------------------------------------------------------------------------------\n");
}


